@@ -39,12 +39,12 @@ modules = [
     "chevereto",
     "cien",
     "civitai",
-    "cohost",
     "comicvine",
     "cyberdrop",
     "danbooru",
     "desktopography",
     "deviantart",
+    "discord",
     "dynastyscans",
     "e621",
     "erome",
@@ -52,12 +52,12 @@ modules = [
     "exhentai",
     "facebook",
     "fanbox",
-    "fanleaks",
     "fantia",
     "fapello",
     "fapachi",
     "flickr",
     "furaffinity",
+    "furry34",
     "fuskator",
     "gelbooru",
     "gelbooru_v01",
@@ -82,6 +82,7 @@ modules = [
     "imgbox",
     "imgth",
     "imgur",
+    "imhentai",
     "inkbunny",
     "instagram",
     "issuu",
@@ -116,6 +117,7 @@ modules = [
     "myportfolio",
     "naver",
     "naverwebtoon",
+    "nekohouse",
     "newgrounds",
     "nhentai",
     "nijie",
@@ -124,6 +126,7 @@ modules = [
     "nsfwalbum",
     "paheal",
     "patreon",
+    "pexels",
     "philomena",
     "photovogue",
     "picarto",
@@ -168,6 +171,7 @@ modules = [
     "tapas",
     "tcbscans",
     "telegraph",
+    "tiktok",
     "tmohentai",
     "toyhouse",
     "tsumino",
@@ -190,10 +194,12 @@ modules = [
     "weasyl",
     "webmshare",
     "webtoons",
+    "weebcentral",
     "weibo",
     "wikiart",
     "wikifeet",
     "wikimedia",
+    "xfolio",
     "xhamster",
     "xvideos",
     "yiffverse",
