@@ -10,7 +10,8 @@
 
 from .common import Extractor
 from .lolisafe import LolisafeAlbumExtractor
-from .. import text, config, exception
+from .. import text, util, config, exception
+import binascii
 import random
 
 if config.get(("extractor", "bunkr"), "tlds"):
@@ -60,6 +61,8 @@ class BunkrAlbumExtractor(LolisafeAlbumExtractor):
     """Extractor for bunkr.si albums"""
     category = "bunkr"
     root = "https://bunkr.si"
+    root_dl = "https://get.bunkrr.su"
+    archive_fmt = "{album_id}_{id|id_url}"
     pattern = BASE_PATTERN + r"/a/([^/?#]+)"
     example = "https://bunkr.si/a/ID"
 
@@ -68,8 +71,14 @@ class BunkrAlbumExtractor(LolisafeAlbumExtractor):
         domain = self.groups[0] or self.groups[1]
         if domain not in LEGACY_DOMAINS:
             self.root = "https://" + domain
+        self.offset = 0
+
+    def skip(self, num):
+        self.offset = num
+        return num
 
     def request(self, url, **kwargs):
+        kwargs["encoding"] = "utf-8"
         kwargs["allow_redirects"] = False
 
         while True:
@@ -80,6 +89,9 @@ class BunkrAlbumExtractor(LolisafeAlbumExtractor):
 
                 # redirect
                 url = response.headers["Location"]
+                if url[0] == "/":
+                    url = self.root + url
+                    continue
                 root, path = self._split(url)
                 if root not in CF_DOMAINS:
                     continue
@@ -105,37 +117,43 @@ class BunkrAlbumExtractor(LolisafeAlbumExtractor):
                             "All Bunkr domains require solving a CF challenge")
 
             # select alternative domain
-            root = "https://" + random.choice(DOMAINS)
+            self.root = root = "https://" + random.choice(DOMAINS)
             self.log.debug("Trying '%s' as fallback", root)
             url = root + path
 
     def fetch_album(self, album_id):
         # album metadata
         page = self.request(self.root + "/a/" + album_id).text
-        title, size = text.split_html(text.extr(
-            page, "<h1", "</span>").partition(">")[2])
-        if "&" in title:
-            title = title.replace(
-                "&lt;", "<").replace("&gt;", ">").replace("&amp;", "&")
+        title = text.unescape(text.unescape(text.extr(
+            page, 'property="og:title" content="', '"')))
 
         # files
-        items = list(text.extract_iter(page, "<!-- item -->", "<!--  -->"))
+        items = list(text.extract_iter(
+            page, '<div class="grid-images_box', "</a>"))
+
         return self._extract_files(items), {
             "album_id"   : album_id,
             "album_name" : title,
-            "album_size" : text.extr(size, "(", ")"),
+            "album_size" : text.extr(
+                page, '<span class="font-semibold">(', ')'),
             "count"      : len(items),
         }
 
     def _extract_files(self, items):
+        if self.offset:
+            items = util.advance(items, self.offset)
+
         for item in items:
             try:
-                url = text.extr(item, ' href="', '"')
-                file = self._extract_file(text.unescape(url))
+                url = text.unescape(text.extr(item, ' href="', '"'))
+                if url[0] == "/":
+                    url = self.root + url
 
+                file = self._extract_file(url)
                 info = text.split_html(item)
-                file["name"] = info[0]
-                file["size"] = info[2]
+                if not file["name"]:
+                    file["name"] = info[-3]
+                file["size"] = info[-2]
                 file["date"] = text.parse_datetime(
                     info[-1], "%H:%M:%S %d/%m/%Y")
 
@@ -147,23 +165,43 @@ class BunkrAlbumExtractor(LolisafeAlbumExtractor):
                 self.log.debug("", exc_info=exc)
 
     def _extract_file(self, webpage_url):
-        response = self.request(webpage_url)
-        page = response.text
-        file_url = (text.extr(page, '<source src="', '"') or
-                    text.extr(page, '<img src="', '"'))
+        page = self.request(webpage_url).text
+        data_id = text.extr(page, 'data-file-id="', '"')
+        referer = self.root_dl + "/file/" + data_id
 
-        if not file_url:
-            webpage_url = text.unescape(text.rextract(
-                page, ' href="', '"', page.rindex("Download"))[0])
-            response = self.request(webpage_url)
-            file_url = text.rextract(response.text, ' href="', '"')[0]
+        url = self.root_dl + "/api/vs"
+        headers = {"Referer": referer}
+        data = self.request(
+            url, method="POST", headers=headers, json={"id": data_id}).json()
+
+        if data.get("encrypted"):
+            file_url = self._decrypt_url(data["url"], data["timestamp"])
+        else:
+            file_url = data["url"]
+
+        file_name = (text.extr(page, 'property="og:title" content="', '"') or
+                     text.extr(page, "<title>", " | Bunkr<"))
+        fallback = text.extr(page, 'property="og:url" content="', '"')
 
         return {
-            "file"          : text.unescape(file_url),
-            "_http_headers" : {"Referer": response.url},
+            "file"          : file_url,
+            "name"          : text.unescape(file_name),
+            "id_url"        : data_id,
+            "_fallback"     : (fallback,) if fallback else (),
+            "_http_headers" : {"Referer": referer},
             "_http_validate": self._validate,
         }
 
+    def _decrypt_url(self, encrypted_b64, timestamp):
+        encrypted_bytes = binascii.a2b_base64(encrypted_b64)
+        key = "SECRET_KEY_{}".format(timestamp // 3600).encode()
+        div = len(key)
+
+        return bytes([
+            encrypted_bytes[i] ^ key[i % div]
+            for i in range(len(encrypted_bytes))
+        ]).decode()
+
     def _validate(self, response):
         if response.history and response.url.endswith("/maintenance-vid.mp4"):
             self.log.warning("File server in maintenance mode")
@@ -179,8 +217,8 @@ class BunkrMediaExtractor(BunkrAlbumExtractor):
     """Extractor for bunkr.si media links"""
     subcategory = "media"
     directory_fmt = ("{category}",)
-    pattern = BASE_PATTERN + r"(/[vid]/[^/?#]+)"
-    example = "https://bunkr.si/v/FILENAME"
+    pattern = BASE_PATTERN + r"(/[fvid]/[^/?#]+)"
+    example = "https://bunkr.si/f/FILENAME"
 
     def fetch_album(self, album_id):
         try:
